/*
 * XMLNewsParser.java
 *
 * Created on February 28, 2002, 7:34 AM
 */
import javax.xml.parsers.*;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
/**
 * This is an XML News Parser which implents the XMLNewsParaser interface.
 * It is implemented using a DOM parsing method
 *
 * @author  andrewm
 * @version 1.0
 */
public class XMLDOMNewsParser implements XMLNewsParser
{
    // String constants for xml tag names in the xml news document
    private final String NEWS_ARTICLE_TAGNAME = "entry";
    private final String NEWS_HEADLINE_TAGNAME = "headline";
    private final String NEWS_HARVESTTIME_TAGNAME = "harvesttime";
    private final String NEWS_URL_TAGNAME = "url";
    
    // Document that is returned after parsing the stream
    private Document document;
    
    // Input stream which should be from a valid xml news source
    private InputStream stream;
    
    /**
     * Public constructor for the XMLDOMNewsParser
     *
     * @param _stream InputStream which should be a valid xml news article
     */
    public XMLDOMNewsParser( InputStream _stream )
    {
	this.stream = _stream;
    }
    
    /**
     * Parses the stream that was passed into the constructor or that was set in
     * the setInputStream method.
     *
     * @return boolean Returns true if the stream was parsed without throwing
     *                 any exceptions.
     */
    public boolean parse()
    {
	// Success flag
	boolean success = false;
	
	// Retrieve an instance of the DocumentBuilderFactory
	DocumentBuilderFactory factory =  DocumentBuilderFactory.newInstance();
	
	try
	{
	    // Create a new DocumentBuilder object
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    
	    // Parse the stream
	    document = builder.parse( stream );
	    
	    // If no exceptions were thrown, set the success flag to true
	    success = true;
	}
	catch( Throwable e )
	{
	    e.printStackTrace();
	}
	finally
	{
	    // Return the success flag
	    return success;
	}
    }
    
    /**
     * Retrieves the number of elements from the DOM tree with the tag name that
     * is passed in.
     *
     * @param elementName Element name or tag name of the desired element
     * @return int Number elements with the given tag name that exist in the DOM tree
     */
    public int getNumElements( String elementName )
    {
	int size = 0;
	
	try
	{
	    // Retrieve the nodes with the given tag name
	    NodeList nodelist = document.getElementsByTagName( elementName );
	    
	    // Set size to the number of elements retrieved
	    size = nodelist.getLength();
	}
	catch( Throwable t )
	{
	    t.printStackTrace();
	}
	finally
	{
	    // Return the size
	    return size;
	}
    }
    
    /**
     * Get the number of articles in the DOM tree
     *
     * @return int Number of news articles in the DOM tree that was created from
     *             parsing the xml news document
     */
    public int getNumArticles()
    {
	// Get the number of elements with the news article tag name
	return getNumElements( NEWS_ARTICLE_TAGNAME );
    }
    
    /**
     * Retrieves elements from an xml news document with the tag name passed in.
     *
     * @param elementName Element or tag name of the desired element
     * @return Nodelist list of nodes from the DOM tree that match the tag name.
     */
    public NodeList getXMLNewsElements( String elementName )
    {
	// List of nodes from the DOM tree
	NodeList nodes = null;
	
	try
	{
	    // Retrieve all the nodes from the tree with the desired tag name
	    nodes = document.getElementsByTagName( elementName );
	}
	catch( Throwable t )
	{
	    t.printStackTrace();
	}
	finally
	{
	    // Return the nodes
	    return nodes;
	}
    }
    
    /**
     * Retrieves all of the xml news articles from the document that was created
     * from parsing the xml stream.  Returns the articles as an enumeration
     *
     * @param maxArticles Maximum number of articles that should be returned.
     * @return Enumeration Enumeration of the articles found.
     */
    public Enumeration getXMLNewsArticles(int maxArticles)
    {
	// Vector to store the news articles
	Vector articles = new Vector();
	
	// DOM parser nodes
	NodeList nodes;
	Node node;
	
	// number of articles to be retrieved
	int numArticles;
	
	// XML news article
	XMLNewsArticle article;
	
	try
	{
	    // Get all of the elements from the tree with the news article tag name
	    nodes = getXMLNewsElements( NEWS_ARTICLE_TAGNAME );
	    
	    // Set the number of articles to be retrieved
	    numArticles = ( nodes.getLength() < maxArticles ) ? nodes.getLength() : maxArticles;
	    
	    // Retrieve the first numArticles elements from the list retrieved above
	    for( int x = 0; x < numArticles; x++ )
	    {
		// Retrieve the next node
		node = nodes.item( x );
		
		// Extract an XMLNewsArticle from the node
		article = getXMLNewsArticle( node );
		
		// Add the article to the vector
		articles.add( article );
	    }
	}
	catch( Throwable t )
	{
	    t.printStackTrace();
	}
	finally
	{
	    // Return the elements of the vector as an enumeration
	    return articles.elements();
	}
    }
    
    /**
     * Creates, populates and returns an XMLNewsArticle object. Extracts the
     * articles data from the DOM node that is passed in.
     *
     * @param node Node of a DOM parser tree
     * @return XMLNewsArticle Article that is populated with the data retrieved
     *         from the node.
     */
    public XMLNewsArticle getXMLNewsArticle( Node node )
    {
	// News article properties
	String headline = null;
	String harvesttime = null;
	String url = null;
	
	// New XMLNewsArticle reference
	XMLNewsArticle article = null;
	
	// Children of the node passed in
	NodeList children = null;
	Node child = null;
	
	try
	{
	    // get the child nodes of the article entry node passed in
	    children = node.getChildNodes();
	    
	    // Loop through all of the child nodes
	    for( int x = 0; x < children.getLength(); x++ )
	    {
		// Get the next child
		child = children.item( x );
		
		// If it is an element node
		if ( child.getNodeType() == Node.ELEMENT_NODE )
		{
		    // If it is a Headline element node
		    if ( child.getNodeName().equals( NEWS_HEADLINE_TAGNAME ) )
		    {
			// Retrieve the text which is in the first child node
			headline = child.getFirstChild().getNodeValue();
		    }
		    // else if it is a harvesttime element node
		    else if ( child.getNodeName().equals( NEWS_HARVESTTIME_TAGNAME ) )
		    {
			// Retrieve the text which is in the first child node
			harvesttime = child.getFirstChild().getNodeValue();
		    }
		    // else if it is a url element node
		    else if ( child.getNodeName().equals( NEWS_URL_TAGNAME ) )
		    {
			// Retrieve the text which is in the first child node
			url = child.getFirstChild().getNodeValue();
		    }
		}
	    }
	    
	    // Create and populate the new XMLNewsArticle
	    article = new XMLNewsArticle( headline, harvesttime, url );
	}
	catch( Throwable t )
	{
	    t.printStackTrace();
	}
	finally
	{
	    // Return the news article
	    return article;
	}
    }
    
    /**
     * setInputStream - sets the input stream to the news stream that is passed
     * in.  This is useful if you want to re-use the same Parser object.  After
     * setting the stream to a new input stream you would then have to call
     * parse again to parse the new input stream.
     *
     * @param _stream InputStream which should contain xml news article text
     */
    public void setInputStream( InputStream _stream )
    {
	this.stream = _stream;
    }
    
    /**
     * main - this is the main method used to test and debug the parser
     *
     * @param argv[] array of command line arguments
     */
    public static void main( String argv[] )
    {
	// Create the content retriever
	ContentRetriever retriever = ContentRetriever.getInstance();
	
	// Retrieve the content from the news server for the specified country
	InputStream xmlStream = retriever.retrieveContent( "http://dcm.cl.uh.edu/yue/courses/xml/notes/general/news.pl?topic=China" );
	
	// Create the XMLNewsParser object
	XMLDOMNewsParser parser = new XMLDOMNewsParser( xmlStream );
	
	// Parse the input stream
	parser.parse();
	
	// Retrieve the first 4 news articles
	Enumeration enums = parser.getXMLNewsArticles( 4 );
	
	XMLNewsArticle article;
	
	// Loop throught the news articles
	while( enums.hasMoreElements() )
	{
	    // get the next news article
	    article = (XMLNewsArticle)enums.nextElement();
	    
	    // Print out the properties of the news article
	    System.out.println( article.getHarvesttime() );
	    System.out.println( article.getHeadline() );
	    System.out.println();
	}
    }
}
